import { Middleware } from 'redux';
import { Json } from '../types';
type HookCallback = (store: any) => void;
export declare class StoreBuilder implements IStoreBuilder {
    private definition;
    private preMiddlewares;
    private postMiddlewares;
    private sagaMiddleware;
    private preBuildHooks;
    private postBuildHooks;
    private autoClear;
    constructor(name: string);
    withInitialState(initialState?: Json): this;
    withPreBuildHook(callback: HookCallback): this;
    withPostBuildHook(callback: HookCallback): this;
    withReducers(reducers: any): this;
    withMiddlewares(middlewares: Middleware | Middleware[]): this;
    withEnhancers(enhancers: any | any[]): this;
    withPreMiddlewares(...middlewares: any): this;
    withPostMiddlewares(...middlewares: any): this;
    withSagas(...sagas: any): this;
    withSagaMonitor(sagaMonitor: any): this;
    withSagaContext(context: any): this;
    clearSagas(): this;
    withDevtoolsExtensions(enable?: boolean): this;
    withAutoClear(): this;
    getSagaMiddleware(): any;
    clear(store: any): void;
    build<T = any>(): T;
}
export interface IStoreBuilder {
    withInitialState: (initialState?: Json) => IStoreBuilder;
    withPreBuildHook: (callback: HookCallback) => IStoreBuilder;
    withPostBuildHook: (callback: HookCallback) => IStoreBuilder;
    withReducers: (reducers: any) => IStoreBuilder;
    withMiddlewares: (middlewares: Middleware | Middleware[]) => IStoreBuilder;
    withEnhancers: (enhancers: any | any[]) => IStoreBuilder;
    withPreMiddlewares: (...middlewares: any) => IStoreBuilder;
    withPostMiddlewares: (...middlewares: any) => IStoreBuilder;
    withSagas: (...sagas: any) => IStoreBuilder;
    withSagaMonitor: (sagaMonitor: any) => IStoreBuilder;
    withSagaContext: (context: any) => IStoreBuilder;
    withAutoClear: () => IStoreBuilder;
    clearSagas: () => IStoreBuilder;
    withDevtoolsExtensions: (enable?: boolean) => IStoreBuilder;
    build<T = any>(): T;
}
export {};
