import { deleteStoreXPath, getStoreXPath, patchStoreXPath, } from '../utils/storage';
import { getActionForNode } from '../utils/actions';
export const getCollection = (action, info) => {
    let nextAction;
    const { nodeName } = info;
    const xpath = `${nodeName}`;
    const collectionItems = getStoreXPath(xpath);
    nextAction = getActionForNode(nodeName, 'setAll', collectionItems);
    return {
        data: collectionItems,
        nextAction,
    };
};
export const patchCollectionItem = (action, info) => {
    const { id, payload = {} } = action;
    const { nodeName } = info;
    const xpath = `${nodeName}.${id}`;
    patchStoreXPath(xpath, payload);
    return {};
};
export const deleteCollectionItem = (action, info) => {
    const { id } = action;
    const { nodeName } = info;
    const xpath = `${nodeName}.${id}`;
    deleteStoreXPath(xpath);
    return {
        data: null,
    };
};
export const addCollectionItem = (action, info) => {
    const { payload = {} } = action;
    const { nodeName } = info;
    const id = action.id ?? payload.id;
    const xpath = `${nodeName}.${id}`;
    patchStoreXPath(xpath, payload);
    const nextAction = getActionForNode(nodeName, 'set', payload);
    return {
        nextAction,
    };
};
export const allVerbs = {
    get: getCollection,
    patch: patchCollectionItem,
    delete: deleteCollectionItem,
    add: addCollectionItem,
    clear: null,
    clearItems: null,
    deleteItem: null,
    getItems: null,
    patchItem: null,
    pop: null,
    popItem: null,
    push: null,
    pushItem: null,
    pushMany: null,
    pushManyItems: null,
    set: null,
    setAll: null,
    setItems: null,
    setMany: null,
};
export const handleAction = async (action, info) => {
    const { verb } = info;
    const verbHandler = allVerbs[verb];
    if (!verbHandler) {
        return false;
    }
    const response = await verbHandler(action, info);
    const { nextAction } = response;
    return {
        nextAction,
        stopPropagation: false,
    };
};
