export class ResponseBuilder {
    constructor(request) {
        this.output = {};
        this.response = {};
        this.transformers = [];
        this.timestampStart = 0;
        this.output = {
            request,
        };
        this.timestampStart = Date.now();
    }
    withAxiosResponse(axiosResponse) {
        this.response = axiosResponse;
        return this;
    }
    withData(data) {
        this.output.data = data;
        return this;
    }
    withIsSuccess(isSuccess) {
        this.output.isSuccess = isSuccess;
        return this;
    }
    withIsError(isError) {
        this.output.isError = isError;
        return this;
    }
    withErrorType(errorType) {
        this.output.errorType = errorType;
        return this;
    }
    withErrorMessage(errorMessage) {
        this.output.errorMessage = errorMessage;
        return this;
    }
    withStatus(status) {
        this.output.status = status;
        return this;
    }
    withTransformer(transformer) {
        if (!transformer) {
            return this;
        }
        this.transformers.push(transformer);
        return this;
    }
    build() {
        if (!this.response) {
            return this.output;
        }
        this.output.status = this.response.status;
        this.output.statusText = this.response.statusText;
        this.output.headers = this.response.headers;
        const now = Date.now();
        this.output.timestampStart = this.timestampStart;
        this.output.timestampEnd = now;
        this.output.duration = now - this.timestampStart;
        this.output.data = this.response.data || this.output.data;
        if (this.transformers.length) {
            this.output.data = this.transformers.reduce((acc, transformer) => {
                return transformer(acc);
            }, this.output.data);
        }
        return this.output;
    }
}
