import { initializeApp } from 'firebase/app';
import { collection, connectFirestoreEmulator, deleteDoc, doc, getDoc, getDocs, getFirestore, onSnapshot, setDoc, updateDoc, writeBatch, } from 'firebase/firestore';
import { ResponseBuilder } from './response';
// Initialize Firebase
let app, db;
export const initFirestore = (firebaseConfig, useEmulator) => {
    app = initializeApp(firebaseConfig, 'redux-connected');
    db = getFirestore(app);
    if (useEmulator) {
        connectFirestoreEmulator(db, 'localhost', 8080);
    }
};
export const setFireStore = (value) => {
    db = value;
};
export const getCollection = async (path) => {
    const response = new ResponseBuilder({});
    const ref = collection(db, path);
    try {
        const snapshot = await getDocs(ref);
        const items = snapshot.docs.map((doc) => doc.data());
        handleSuccess(response, items);
    }
    catch (err) {
        handleError(response, err);
    }
    return response.build();
};
export const addToCollection = async (path, item) => {
    const response = new ResponseBuilder({});
    const ref = collection(db, path);
    const docRef = doc(ref, item.id);
    try {
        await setDoc(docRef, item);
        handleSuccess(response, item);
    }
    catch (err) {
        handleError(response, err);
    }
    return response.build();
};
export const removeCollection = async (path) => {
    const response = new ResponseBuilder({});
    const ref = collection(db, path);
    try {
        const snapshot = await getDocs(ref);
        const items = snapshot.docs.map((doc) => doc.data());
        handleSuccess(response, items);
    }
    catch (err) {
        handleError(response, err);
    }
    return response.build();
};
export const addToCollectionBatch = async (path, item) => {
    const response = new ResponseBuilder({});
    const ref = collection(db, path);
    const batch = writeBatch(db);
    item.forEach((item) => {
        const docRef = doc(ref, item.id);
        batch.set(docRef, item);
    });
    try {
        await batch.commit();
        handleSuccess(response, item);
    }
    catch (err) {
        handleError(response, err);
    }
    return response.build();
};
export const getCollectionItem = async (path) => {
    const response = new ResponseBuilder({});
    const ref = doc(db, path);
    try {
        const docSnap = await getDoc(ref);
        const data = docSnap.data();
        handleSuccess(response, data);
    }
    catch (err) {
        handleError(response, err);
    }
    return response.build();
};
export const setCollectionItem = async (path, value) => {
    const response = new ResponseBuilder({});
    const ref = doc(db, path);
    try {
        await setDoc(ref, value);
        handleSuccess(response, value);
    }
    catch (err) {
        console.log('err =>', err);
        handleError(response, err);
    }
    return response.build();
};
export const updateCollectionItem = async (path, change) => {
    const response = new ResponseBuilder({});
    const ref = doc(db, path);
    try {
        await updateDoc(ref, change);
        const data = await getCollectionItem(path);
        handleSuccess(response, data);
    }
    catch (err) {
        console.log('err =>', err);
        handleError(response, err);
    }
    return response.build();
};
export const removeCollectionItem = async (path) => {
    const response = new ResponseBuilder({});
    const ref = doc(db, path);
    try {
        await deleteDoc(ref);
        handleSuccess(response, {});
    }
    catch (err) {
        handleError(response, err);
    }
    return response.build();
};
export const listenToCollection = (path, callback) => {
    const ref = collection(db, path);
    return onSnapshot(ref, (snapshot) => {
        const change = snapshot.docChanges()[0];
        const item = change.doc.data();
        callback(item);
    });
};
export const setSingle = async (path, change) => {
    const response = new ResponseBuilder({});
    const ref = doc(db, path);
    try {
        await setDoc(ref, change);
        const data = await getSingle(path);
        handleSuccess(response, data);
    }
    catch (err) {
        console.log('err =>', err);
        handleError(response, err);
    }
    return response.build();
};
export const getSingle = async (path) => {
    const response = new ResponseBuilder({});
    const ref = doc(db, path);
    try {
        const docSnap = await getDoc(ref);
        const data = docSnap.data();
        handleSuccess(response, data);
    }
    catch (err) {
        handleError(response, err);
    }
    return response.build();
};
export const handleSuccess = (response, data) => {
    response
        .withData(data) //
        .withIsSuccess(true);
};
export const handleError = (response, err) => {
    response
        .withErrorMessage(err.message) //
        .withIsError(true)
        .withIsSuccess(false);
};
