import { get, set } from 'lodash';
export const getString = (key) => {
    return localStorage.getItem(key);
};
export const setString = (key, value) => {
    localStorage.setItem(key, value);
};
export const removeString = (key) => {
    localStorage.removeItem(key);
};
export const getJson = (key) => {
    const value = localStorage.getItem(key);
    return value ? JSON.parse(value) : null;
};
export const setJson = (key, value) => {
    const valueToStore = JSON.stringify(value);
    setString(key, valueToStore);
};
export const patchJson = (key, value) => {
    const valueToStore = JSON.stringify({ ...getJson(key), ...value });
    setString(key, valueToStore);
};
export const getStore = () => {
    return getJson('STORE') || {};
};
export const setStore = (value) => {
    setJson('STORE', value);
};
export const getStoreXPath = (xpath) => {
    const store = getStore();
    return get(store, xpath);
};
export const patchStoreXPath = (xpath, change) => {
    const store = getStore();
    const value = get(store, xpath);
    set(store, xpath, {
        ...value,
        ...change,
    });
    setStore(store);
};
export const deleteStoreXPath = (xpath) => {
    const store = getStore();
    const parts = xpath.split('.');
    const last = parts.pop();
    const xpathWithoutLast = parts.join('.');
    const value = get(store, xpathWithoutLast);
    if (!last || !(last in value)) {
        return;
    }
    delete value[last];
    set(store, xpathWithoutLast, value);
    setStore(store);
};
